package main

import (
	"context"
	"crypto/rand"
	"encoding/base64"
	"net/http"
	"os"
	"os/signal"
	"sync"
	"syscall"

	restserver "code.fbi.h-da.de/danet/quant/etsi014/go/rest/etsi/server/go"
	"github.com/google/uuid"
	"github.com/sirupsen/logrus"
)

type ETSI14RESTService struct {
	serviceAddress string
}

var keyStoreMutex sync.RWMutex
var keyStore = make(map[string]string, 0)

func NewETSI14RESTService(serviceAdress string) *ETSI14RESTService {
	return &ETSI14RESTService{
		serviceAddress: serviceAdress,
	}
}

func SetupETSI14RestServer(etsi14RESTService *ETSI14RESTService) {
	etsi14APIController := restserver.NewDefaultAPIController(etsi14RESTService)

	router := restserver.NewRouter(etsi14APIController)
	// NOTE check again, currently ignoring
	go http.ListenAndServe(etsi14RESTService.serviceAddress, router) //nolint:errcheck
	logrus.Info("Server started")
}

// GetKey - TBD.
func (s *ETSI14RESTService) GetKey(ctx context.Context, slaveSAEID string, number int64, size int64) (restserver.ImplResponse, error) {
	// TODO - update GetKey with the required logic for this service method.
	// Add api_default_service.go to the .openapi-generator-ignore to avoid overwriting this service implementation when updating open api generation.

	// TODO: Uncomment the next line to return response Response(200, KeyContainer{}) or use other options such as http.Ok ...
	// return Response(200, KeyContainer{}), nil

	// TODO: Uncomment the next line to return response Response(400, {}) or use other options such as http.Ok ...
	// return Response(400, nil),nil

	// TODO: Uncomment the next line to return response Response(401, {}) or use other options such as http.Ok ...
	// return Response(401, nil),nil

	// TODO: Uncomment the next line to return response Response(503, {}) or use other options such as http.Ok ...
	// return Response(503, nil),nil

	keyContainer := &restserver.KeyContainer{}

	for range number {
		keyId := uuid.New().String()

		b := make([]byte, 32)
		_, err := rand.Read(b)
		if err != nil {
			logrus.Errorf("Error generating random key: %v", err)
		}

		keyAsBase64String := base64.StdEncoding.EncodeToString(b)

		keyContainer.Keys = append(keyContainer.Keys, restserver.KeyContainerKeysInner{
			KeyID: keyId,
			Key:   keyAsBase64String,
		})
		keyStoreMutex.Lock()
		keyStore[keyId] = keyAsBase64String
		keyStoreMutex.Unlock()

		logrus.Info("Generated key with ID: ", keyId)
	}

	return restserver.Response(http.StatusOK, keyContainer), nil
}

// GetKeyPost -.
func (s *ETSI14RESTService) GetKeyPost(ctx context.Context, slaveSAEID string, keyRequest restserver.KeyRequest) (restserver.ImplResponse, error) {
	// TODO - update GetKeyPost with the required logic for this service method.
	// Add api_default_service.go to the .openapi-generator-ignore to avoid overwriting this service implementation when updating open api generation.

	// TODO: Uncomment the next line to return response Response(200, KeyContainer{}) or use other options such as http.Ok ...
	// return Response(200, KeyContainer{}), nil

	// TODO: Uncomment the next line to return response Response(400, {}) or use other options such as http.Ok ...
	// return Response(400, nil),nil

	// TODO: Uncomment the next line to return response Response(401, {}) or use other options such as http.Ok ...
	// return Response(401, nil),nil

	// TODO: Uncomment the next line to return response Response(503, {}) or use other options such as http.Ok ...
	// return Response(503, nil),nil

	keyContainer := &restserver.KeyContainer{}
	for range keyRequest.Number {
		keyId := uuid.New().String()

		b := make([]byte, 32)
		_, err := rand.Read(b)
		if err != nil {
			logrus.Errorf("Error generating random key: %v", err)
		}

		keyAsBase64String := base64.StdEncoding.EncodeToString(b)

		keyContainer.Keys = append(keyContainer.Keys, restserver.KeyContainerKeysInner{
			KeyID: keyId,
			Key:   keyAsBase64String,
		})
		keyStoreMutex.Lock()
		keyStore[keyId] = keyAsBase64String
		keyStoreMutex.Unlock()

		logrus.Info("Generated key with ID: ", keyId)
	}

	return restserver.Response(http.StatusOK, keyContainer), nil
}

// GetKeyWithIds - TBD.
func (s *ETSI14RESTService) GetKeyWithIds(ctx context.Context, masterSAEID string, keyID string) (restserver.ImplResponse, error) {
	// TODO - update GetKeyWithIds with the required logic for this service method.
	// Add api_default_service.go to the .openapi-generator-ignore to avoid overwriting this service implementation when updating open api generation.

	// TODO: Uncomment the next line to return response Response(200, KeyContainer{}) or use other options such as http.Ok ...
	// return Response(200, KeyContainer{}), nil

	// TODO: Uncomment the next line to return response Response(400, {}) or use other options such as http.Ok ...
	// return Response(400, nil),nil

	// TODO: Uncomment the next line to return response Response(401, {}) or use other options such as http.Ok ...
	// return Response(401, nil),nil

	// TODO: Uncomment the next line to return response Response(503, {}) or use other options such as http.Ok ...
	// return Response(503, nil),nil

	keyStoreMutex.RLock()
	key, ok := keyStore[keyID]
	if !ok {
		logrus.Errorf("Key with ID %s not found", keyID)
	}
	keyStoreMutex.RUnlock()

	keyContainer := &restserver.KeyContainer{}
	keyContainer.Keys = append(keyContainer.Keys, restserver.KeyContainerKeysInner{
		KeyID: keyID,
		Key:   key,
	})

	return restserver.Response(http.StatusOK, keyContainer), nil
}

// GetKeyWithIdsPost - TBD.
func (s *ETSI14RESTService) GetKeyWithIdsPost(ctx context.Context, masterSAEID string, keyIdsRequest restserver.KeyIdsRequest) (restserver.ImplResponse, error) {
	// TODO - update GetKeyWithIdsPost with the required logic for this service method.
	// Add api_default_service.go to the .openapi-generator-ignore to avoid overwriting this service implementation when updating open api generation.

	// TODO: Uncomment the next line to return response Response(200, KeyContainer{}) or use other options such as http.Ok ...
	// return Response(200, KeyContainer{}), nil

	// TODO: Uncomment the next line to return response Response(400, {}) or use other options such as http.Ok ...
	// return Response(400, nil),nil

	// TODO: Uncomment the next line to return response Response(401, {}) or use other options such as http.Ok ...
	// return Response(401, nil),nil

	// TODO: Uncomment the next line to return response Response(503, {}) or use other options such as http.Ok ...
	// return Response(503, nil),nil

	keyStoreMutex.RLock()
	defer keyStoreMutex.RUnlock()

	keyContainer := &restserver.KeyContainer{}
	for _, inner := range keyIdsRequest.KeyIDs {
		logrus.Infof("Request for Key with ID: %s", inner.KeyID)
		key, ok := keyStore[inner.KeyID]
		if !ok {
			logrus.Errorf("Key with ID %s not found", inner.KeyID)
		}

		keyContainer.Keys = append(keyContainer.Keys, restserver.KeyContainerKeysInner{
			KeyID: inner.KeyID,
			Key:   key,
		})
	}

	return restserver.Response(http.StatusOK, keyContainer), nil
}

// GetStatus - TBD.
func (s *ETSI14RESTService) GetStatus(ctx context.Context, slaveSAEID string) (restserver.ImplResponse, error) {
	// TODO - update GetStatus with the required logic for this service method.
	// Add api_default_service.go to the .openapi-generator-ignore to avoid overwriting this service implementation when updating open api generation.

	// TODO: Uncomment the next line to return response Response(200, Status{}) or use other options such as http.Ok ...
	// return Response(200, Status{}), nil

	// TODO: Uncomment the next line to return response Response(400, {}) or use other options such as http.Ok ...
	// return Response(400, nil),nil

	// TODO: Uncomment the next line to return response Response(401, {}) or use other options such as http.Ok ...
	// return Response(401, nil),nil

	// TODO: Uncomment the next line to return response Response(503, {}) or use other options such as http.Ok ...
	// return Response(503, nil),nil

	status := restserver.Status{
		SourceKMEID:      "AAAABBBBCCCCDDDD",
		TargetKMEID:      "EEEEFFFFGGGGHHHH",
		MasterSAEID:      "IIIIJJJJKKKKLLLL",
		SlaveSAEID:       "MMMMNNNNOOOOPPPP",
		KeySize:          352,
		StoredKeyCount:   25000,
		MaxKeyCount:      100000,
		MaxKeyPerRequest: 128,
		MaxKeySize:       1024,
		MinKeySize:       64,
		MaxSAEIDCount:    0,
	}

	resp := restserver.Response(http.StatusOK, status)

	return resp, nil
}

func main() {
	etsi14RESTService := NewETSI14RESTService(":1414")
	SetupETSI14RestServer(etsi14RESTService)

	stopChan := make(chan os.Signal, 1)
	signal.Notify(stopChan, os.Interrupt, syscall.SIGTERM)

	<-stopChan
}
