package kmsHandler

import (
	"fmt"

	"code.fbi.h-da.de/danet/gnmi-target/handler"
	"code.fbi.h-da.de/danet/quant/goKMS/kms"
	"code.fbi.h-da.de/danet/quant/goKMS/kms/event"
	gnmitargetygot "code.fbi.h-da.de/danet/quant/goKMS/model"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	log "github.com/sirupsen/logrus"
)

type CreateRouteHandler struct {
	handler.DefaultPathHandler
	kms    *kms.KMS
	events <-chan event.Event
}

func NewCreateRouteHandler(kms *kms.KMS) *CreateRouteHandler {
	return &CreateRouteHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "kms-create-route-handler",
			Paths: map[string]struct{}{
				"/create-route-requests": {},
			},
		},
		kms: kms,
	}
}

func (yh *CreateRouteHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	var err error
	yh.events, err = yh.kms.EventBus().Subscribe(event.CREATE_ROUTE)
	if err != nil {
		return err
	}

	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	newConfig.GetOrCreateCreateRouteRequests()

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return err
	}

	yh.Config.Data = newConfig

	// Start the go routine that takes care of any update from the kms
	go func() {
		for {
			select {
			case e := <-yh.events:
				log.Println("Update for create route requests.")

				createRouteEvent, ok := e.(*event.CreateRouteEvent)
				if !ok {
					log.Errorf("Type assertion failed, expected: %T, got: %T", (*event.CreateRouteEvent)(nil), e)
					break
				}
				diff, err := yh.createRouteRequest(createRouteEvent)
				if err != nil {
					log.Errorf("Error within interface subscription goroutine; %v", err)
					break
				}
				if err := yh.PublishToSubs(diff); err != nil {
					log.Errorf("Error within interface subscription goroutine; %v", err)
				}
			}
		}
	}()

	return nil
}

func (yh *CreateRouteHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	return fmt.Errorf("Not implemented")
}

func (yh *CreateRouteHandler) createRouteRequest(event *event.CreateRouteEvent) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confCreateRouteRequests := newConfig.GetOrCreateCreateRouteRequests()
	confCreateRouteRequest, err := confCreateRouteRequests.NewCreateRouteRequest(event.KmsPathId)
	if err != nil {
		return nil, err
	}

	confCreateRouteRequest.KmsPathId = ygot.String(event.KmsPathId)
	confCreateRouteRequest.RequestStatus = gnmitargetygot.Temp_CreateRouteRequests_CreateRouteRequest_RequestStatus_PENDING
	confCreateRouteRequest.RemoteUkmsId = ygot.String(event.RemoteUkmsId)

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
