package system

import (
	"fmt"
	"time"

	"code.fbi.h-da.de/danet/gnmi-target/handler"
	gnmitargetygot "code.fbi.h-da.de/danet/quant/goKMS/model"
	"code.fbi.h-da.de/danet/quant/goKMS/osclient"
	"github.com/openconfig/gnmi/proto/gnmi"
	"github.com/openconfig/ygot/ygot"
	gopshost "github.com/shirou/gopsutil/host"
)

// StateHandler is the implementation of a gnmitarget.PathHandler.
type StateHandler struct {
	handler.DefaultPathHandler
	osClient osclient.Osclient
}

func NewStateHandler() *StateHandler {
	return &StateHandler{
		DefaultPathHandler: handler.DefaultPathHandler{
			Name: "openconfig-memory-handler",
			Paths: map[string]struct{}{
				"/system/state": {},
			},
		},
		osClient: osclient.NewOsClient(),
	}
}

func (yh *StateHandler) Init(config *handler.Config, publishToSubsFunc func([]*gnmi.Notification) error) error {
	yh.Config = config
	yh.PublishToSubs = publishToSubsFunc

	_, err := yh.updateOrCreateState(yh.osClient)
	if err != nil {
		return err
	}
	return nil
}

func (yh *StateHandler) Update(c ygot.ValidatedGoStruct, jobs []*gnmi.Update) error {
	fmt.Println("Update request received for ", yh.Name)
	return nil
}

func (yh *StateHandler) updateOrCreateState(os osclient.Osclient) ([]*gnmi.Notification, error) {
	yh.Config.Lock()
	defer yh.Config.Unlock()

	copyCurrentConfig, err := ygot.DeepCopy(yh.Config.Data)
	if err != nil {
		return nil, err
	}

	newConfig, ok := copyCurrentConfig.(*gnmitargetygot.Gnmitarget)
	if !ok {
		return nil, fmt.Errorf("Wrong type, exptected: %T, got: %T", (*gnmitargetygot.OpenconfigInterfaces_Interfaces)(nil), copyCurrentConfig)
	}

	confSystem := newConfig.GetOrCreateSystem()

	if state := confSystem.GetOrCreateState(); state != nil {
		state.CurrentDatetime = ygot.String(time.Now().Format(time.RFC3339))
		bootTime, err := gopshost.BootTime()
		if err != nil {
			return nil, err
		}
		state.BootTime = ygot.Uint64(bootTime)

		// Read version of the operating system
		osVersion, err := os.GetSoftwareVersion()
		if err != nil {
			return nil, fmt.Errorf("Failed to get the current software version of the operating system; err: %w", err)
		}
		state.SoftwareVersion = ygot.String(osVersion)
	}

	//validate struct
	if err := newConfig.Validate(); err != nil {
		return nil, err
	}

	notifications, err := ygot.DiffWithAtomic(yh.Config.Data, newConfig)
	if err != nil {
		return nil, err
	}

	yh.Config.Data = newConfig

	return notifications, nil
}
