package client

import (
	"bytes"
	"encoding/json"
	"net/http"

	"code.fbi.h-da.de/danet/quant/goKMS/kms/crypto"
	"github.com/sirupsen/logrus"
)

type CkmsAkmsClient struct {
	url string
}

func NewCkmsAkmsClient(url string) *CkmsAkmsClient {
	return &CkmsAkmsClient{
		url: url,
	}
}

type PushKSAKeyRequest struct {
	RequestID string          `json:"request_ID"`
	ProcessID string          `json:"process_ID"`
	KSAKeys   []crypto.KSAKey `json:"ksa_keys"`
}

func (c *CkmsAkmsClient) SendKSAKeysToRequestingInstances(requestID string, processID string, ksaKeys []crypto.KSAKey) error {
	pushRequest := PushKSAKeyRequest{
		RequestID: requestID,
		ProcessID: processID,
		KSAKeys:   ksaKeys,
	}

	jsonData, err := json.Marshal(pushRequest)
	if err != nil {
		logrus.Errorf("Error marshaling JSON: %s", err)
		return err
	}

	resp, err := http.Post(c.url, "application/json", bytes.NewBuffer(jsonData))
	if err != nil {
		logrus.Errorf("Error sending POST request: %s", err)
		logrus.Errorf("Tried to send request: %s to url: %s", jsonData, c.url)
		return err
	}
	err = resp.Body.Close()
	if err != nil {
		logrus.Errorf("Error closing response body: %s", err)
	}

	if resp.StatusCode != http.StatusNoContent {
		logrus.Errorf("Unexpected response status code: %d", resp.StatusCode)
		logrus.Errorf("Tried to send request: %s to url: %s", jsonData, c.url)
		return err
	}

	logrus.Infof("Successfully sent request: %s to url: %s", jsonData, c.url)
	return nil
}
