package restclient

import (
	"context"
	"net/http"

	restclient "code.fbi.h-da.de/danet/quant/etsi014/go/rest/etsi/client"
	"github.com/sirupsen/logrus"
)

type ClientImpl struct {
	restClient *restclient.APIClient
}

func NewClientImpl(clientConf *restclient.Configuration) (*ClientImpl, error) {
	client := restclient.NewAPIClient(clientConf)

	return &ClientImpl{
		restClient: client,
	}, nil
}

func (c *ClientImpl) GetStatus(slaveSAEID string) (*restclient.Status, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetStatus(context.Background(), slaveSAEID)
	return req.Execute()
}

func (c *ClientImpl) GetKey(slaveSAEID string) (*restclient.KeyContainer, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetKey(context.Background(), slaveSAEID).Number(1).Size(256)
	return req.Execute()
}

func (c *ClientImpl) GetKeyPost(slaveSAEID string) (*restclient.KeyContainer, *http.Response, error) {
	postReq := c.restClient.DefaultAPI.GetKeyPost(context.Background(), slaveSAEID)
	postReq.KeyRequest(restclient.KeyRequest{
		Number:            restclient.PtrInt64(1),
		Size:              restclient.PtrInt64(256),
		ExtensionOptional: []map[string]string{},
	})

	return postReq.Execute()
}

func (c *ClientImpl) GetKeyWithId(masterSAEID, keyID string) (*restclient.KeyContainer, *http.Response, error) {
	req := c.restClient.DefaultAPI.GetKeyWithIds(context.Background(), masterSAEID).KeyID(keyID)

	return req.Execute()
}

func (c *ClientImpl) GetKeyWithIdPost(masterSAEID string, keyIDs []restclient.KeyIDsRequestKeyIDsInner) (*restclient.KeyContainer, *http.Response, error) {
	keyIDsRequest := restclient.NewKeyIDsRequest()
	keyIDsRequest.SetKeyIDs(keyIDs)

	postReq := c.restClient.DefaultAPI.GetKeyWithIdsPost(context.Background(), masterSAEID).KeyIDsRequest(*keyIDsRequest)

	logrus.Info(keyIDsRequest)

	return postReq.Execute()
}
