package event

import "fmt"

type EventBus struct {
	subscribers map[Topic]map[chan<- Event]struct{}
}

func NewEventBus() *EventBus {
	return &EventBus{
		subscribers: make(map[Topic]map[chan<- Event]struct{}),
	}
}

func (b *EventBus) Subscribe(topic Topic) (<-chan Event, error) {
	newSubChan := make(chan Event)

	subs, ok := b.subscribers[topic]
	if !ok {
		initialSub := map[chan<- Event]struct{}{
			newSubChan: {},
		}
		b.subscribers[topic] = initialSub
	} else if subs != nil {
		subs[newSubChan] = struct{}{}
	} else {
		return nil, fmt.Errorf("could not subscribe to topic: %d", topic)
	}

	return newSubChan, nil
}

func (b *EventBus) Publish(event Event) error {
	subs, ok := b.subscribers[event.Topic()]
	if !ok {
		return fmt.Errorf("There are no active subscribers for topic: %d", event.Topic())
	}
	for sub := range subs {
		sub <- event
	}
	return nil
}
