package receiver

import (
	"fmt"
	"sync"

	"github.com/google/uuid"
	"github.com/sirupsen/logrus"
)

type Receiver struct {
	mu        sync.RWMutex
	Receivers map[uuid.UUID]chan<- struct{}
}

func (r *Receiver) RequestReceiverChannel(pathId uuid.UUID) (<-chan struct{}, error) {
	newSubChan := make(chan struct{})

	r.mu.Lock()
	defer r.mu.Unlock()
	r.Receivers[pathId] = newSubChan

	return newSubChan, nil
}

func (r *Receiver) RemoveReceiver(pathId uuid.UUID) error {
	r.mu.Lock()
	defer r.mu.Unlock()

	if channel, ok := r.Receivers[pathId]; ok {
		close(channel)
		delete(r.Receivers, pathId)
	} else {
		return fmt.Errorf("There are no active subscribers for pathId: %s", pathId)
	}

	logrus.Debugf("Current active receivers after removing receiver for pathId: %s; Receivers: %v", pathId, r.Receivers)
	return nil
}

func (r *Receiver) InformReceiver(pathId uuid.UUID) error {
	r.mu.RLock()
	defer r.mu.RUnlock()

	if receiver, ok := r.Receivers[pathId]; ok {
		go func() { receiver <- struct{}{} }()
	} else {
		return fmt.Errorf("There are no active subscribers for pathId: %s", pathId)
	}
	return nil
}
