package util

import (
	"bytes"
	"fmt"
	"net/http"

	"github.com/hashicorp/go-multierror"
)

type RemoteKMS struct {
	Id      string
	Address string
	Port    uint16
}

func RandomItemFromMap[T comparable, M any](m map[T]M) (T, M, error) {
	for key, item := range m {
		return key, item, nil
	}

	var k T
	var i M
	return k, i, fmt.Errorf("Could not return a item from map %v", m)
}

func RandomItemFromMapAndRemove[T comparable, M any](m map[T]M) (T, M, error) {
	for key, item := range m {
		delete(m, key)
		return key, item, nil
	}

	var k T
	var i M
	return k, i, fmt.Errorf("Could not return a random item from map %v and delete it", m)
}

func KeysOfMap[T comparable, M any](m map[T]M) []T {
	keys := make([]T, len(m))

	i := 0
	for k := range m {
		keys[i] = k
		i++
	}

	return keys
}

// NOTE: For demo purpose only.
func SendKmsInfoMessage(url string, json []byte) (err error) {
	request, err := http.NewRequest("POST", url, bytes.NewBuffer(json))
	if err != nil {
		return err
	}
	request.Header.Set("Content-Type", "application/json; charset=UTF-8")
	client := &http.Client{}
	response, err := client.Do(request)
	if err != nil {
		if closeError := response.Body.Close(); closeError != nil {
			err = multierror.Append(err, closeError)
		}
		return err
	}
	defer func() {
		if closeError := response.Body.Close(); closeError != nil {
			err = multierror.Append(err, closeError)
		}
	}()
	return nil
}

type KMSInfo struct {
	Name             string `json:"Name"`
	EncryptedMessage string `json:"EncryptedMessage"`
	DecryptedMessage string `json:"DecryptedMessage"`
	Key              string `json:"Key"`
}
