package integration_tests

import (
	"bytes"
	"encoding/json"
	"fmt"
	"net/http"
	"os"
	"strings"
	"testing"
	"time"

	integration_test_utils "code.fbi.h-da.de/danet/quant/integration-tests/code/integrationTestUtils"
	"github.com/stretchr/testify/assert"
)

func TestETSI14GetKey(t *testing.T) { //nolint:gocyclo
	kms2URL := "127.0.0.1:7031"
	kms2URL_ENV := os.Getenv("INTEGRATION_TEST_KMS2_URL")
	if kms2URL_ENV != "" {
		kms2URL = kms2URL_ENV
	}
	kms1URL := "127.0.0.1:7030"
	kms1URL_ENV := os.Getenv("INTEGRATION_TEST_KMS1_URL")
	if kms1URL_ENV != "" {
		kms1URL = kms1URL_ENV
	}
	kms1ETSI14URL := "127.0.0.1:1414"
	kms1ETSI14URL_ENV := os.Getenv("INTEGRATION_TEST_KMS1_ETSI14_URL")
	if kms1ETSI14URL_ENV != "" {
		kms1ETSI14URL = kms1ETSI14URL_ENV
	}
	kms2ETSI14URL := "127.0.0.1:1415"
	kms2ETSI14URL_ENV := os.Getenv("INTEGRATION_TEST_KMS2_ETSI14_URL")
	if kms2ETSI14URL_ENV != "" {
		kms2ETSI14URL = kms2ETSI14URL_ENV
	}

	url := fmt.Sprintf("http://%s/api/v1/keys/slave_SAE_ID/enc_keys", kms1ETSI14URL)
	data := ETSI14RequestData{
		Number: 1,
	}

	jsonData, err := json.Marshal(data)
	if err != nil {
		fmt.Println(err)
		return
	}

	var respKms1 *http.Response
	go func() {
		respKms1, err = http.Post(url, "application/json", bytes.NewBuffer(jsonData))
		if err != nil {
			t.Errorf("Error making HTTP request: %s", err)
			return
		}

		if respKms1.StatusCode != http.StatusOK {
			t.Errorf("Expected status code 200 OK, but got %d", respKms1.StatusCode)
		}
	}()

	time.Sleep(time.Duration(2) * time.Second)

	output, err := integration_test_utils.GnmicCommand(kms1URL, "get", "--path", "create-route-requests")
	if err != nil {
		t.Errorf("Error getting create-route-requests: %s; %s", err, output)
	}

	output = strings.Split(output, "\"kms-path-id\": \"")[1]
	output = strings.Split(output, "\",")[0]

	path := fmt.Sprintf("key-routing-sessions/routing-sessions[path-id=%s]", output)

	config01 := integration_test_utils.BuildKms1RoutingConfig(path, output)

	config02 := integration_test_utils.BuildKms2RoutingConfig(path, output)

	argsKMS1 := []string{"set"}
	for _, update := range config01 {
		argsKMS1 = append(argsKMS1, "--update-path", update.Path, "--update-value", update.JSON)
	}

	argsKMS2 := []string{"set"}
	for _, update := range config02 {
		argsKMS2 = append(argsKMS2, "--update-path", update.Path, "--update-value", update.JSON)
	}

	output, err = integration_test_utils.GnmicCommand(kms2URL, argsKMS2...)
	if err != nil {
		t.Errorf("Error setting routing-session: %s; %s", err, output)
	}

	output, err = integration_test_utils.GnmicCommand(kms1URL, argsKMS1...)
	if err != nil {
		t.Errorf("Error setting routing-session: %s; %s", err, output)
	}

	time.Sleep(time.Duration(2) * time.Second)

	kms1KeyContainer, err := getKeyContainerFromResponse(respKms1)
	if err != nil {
		t.Errorf("Error getting Key Container from resp: %v, err: %v", respKms1, err)
	}

	assert.NotNil(t, kms1KeyContainer.Keys)

	url = fmt.Sprintf("http://%s/api/v1/keys/master_SAE_ID/dec_keys", kms2ETSI14URL)
	data = ETSI14RequestData{
		KeyIDs: []KeyIdsRequestKeyIdsInner{
			{
				KeyID: kms1KeyContainer.Keys[0].KeyID,
			},
		},
	}

	jsonData, err = json.Marshal(data)
	if err != nil {
		fmt.Println(err)
		return
	}

	var respKms2 *http.Response
	go func() {
		respKms2, err = http.Post(url, "application/json", bytes.NewBuffer(jsonData))
		if err != nil {
			t.Errorf("Error making HTTP request: %s", err)
			return
		}

		if respKms2.StatusCode != http.StatusOK {
			t.Errorf("Expected status code 200 OK, but got %d", respKms2.StatusCode)
		}
	}()

	time.Sleep(time.Duration(2) * time.Second)

	kms2KeyContainer, err := getKeyContainerFromResponse(respKms2)
	if err != nil {
		t.Errorf("Error getting Key Container from resp: %v, err: %v", respKms1, err)
	}

	assert.NotNil(t, kms2KeyContainer.Keys)

	assert.Equal(t, kms1KeyContainer, kms2KeyContainer)
}
