package integration_tests

import (
	"encoding/json"
	"io"
	"net/http"
)

type ETSI14RequestData struct {
	Number int64                      `json:"number,omitempty"`
	KeyIDs []KeyIdsRequestKeyIdsInner `json:"key_IDs,omitempty"`
}

type KeyIdsRequestKeyIdsInner struct {
	KeyID string `json:"key_ID"`
}

type ETSI14ResponseData struct {
	Keys []KeyContainerKeysInner `json:"Keys,omitempty"`
}

type KeyContainerKeysInner struct {
	KeyID string `json:"key_ID,omitempty"`

	Key string `json:"key,omitempty"`
}

func getKeyContainerFromResponse(kmsResp *http.Response) (*ETSI14ResponseData, error) {
	defer kmsResp.Body.Close() //nolint:errcheck
	body, err := io.ReadAll(kmsResp.Body)
	if err != nil {
		return nil, err
	}

	kmsKeyContainer := &ETSI14ResponseData{}
	err = json.Unmarshal([]byte(body), kmsKeyContainer)
	if err != nil {
		return nil, err
	}

	return kmsKeyContainer, nil
}
