// This package aims at emulating a quantum link and is extendable to different models
// One can use most of the sourc code of emu-prng and reuse it.
// To add a different quantum module one should only modify the RandomNumberSource function

package quantumlayer

type QuantumLayerBulkKey struct {
	BulkKeyId     int64  // the unique ID of this bulk of keys
	BulkKeyLength int    // the length, counted in bytes, of bulkKey
	BulkKey       []byte // the bulk key
}

type NumberLayer interface {
	GetBatch() []byte // allows to retrieve the current available batch of numbers
	GetBulk() (QuantumLayerBulkKey, error)
	receiveNumbers(chan []byte)
}

type QuantumLayer interface {
	Start()
	RandomNumberSource // interface for the random number source
}

type RandomNumberSource interface {
	InitializeRandomGenerator() error
	GenerateRandomBytes() (randomBytes []byte, length int, err error)
}
