package randomnumbergenerator

import (
	"math/rand"
	"time"

	"github.com/sirupsen/logrus"
)

// RandomNumberGenerator is a struct that generates random numbers.
type RandomNumberGenerator struct {
	randomBytesMinimumLength    int
	randomBytesMaximumLength    int
	seed                        int64
	sleepTimerForGenerationInMs int // Simulates time to generate a random number
	randomGenerator             *rand.Rand
}

func NewRandomNumberGenerator(randomBytesMinimumLength int, randomBytesMaximumLength int, seed int64, sleepTimerForGenerationInMs int) *RandomNumberGenerator {
	return &RandomNumberGenerator{
		randomBytesMinimumLength:    randomBytesMinimumLength,
		randomBytesMaximumLength:    randomBytesMaximumLength,
		seed:                        seed,
		sleepTimerForGenerationInMs: sleepTimerForGenerationInMs,
	}
}

// InitializeRandomNumbers initializes the random numbers.
func (rng *RandomNumberGenerator) InitializeRandomGenerator() error {
	rng.randomGenerator = rand.New(rand.NewSource(rng.seed))
	return nil
}

// GenerateRandomNumbers generates random numbers.
func (rng *RandomNumberGenerator) GenerateRandomBytes() ([]byte, int, error) {
	time.Sleep(time.Duration(rng.sleepTimerForGenerationInMs) * time.Millisecond)

	length := rng.randomGenerator.Intn(rng.randomBytesMaximumLength-rng.randomBytesMinimumLength+1) + rng.randomBytesMinimumLength
	b := make([]byte, length)
	length, err := rng.randomGenerator.Read(b)
	if err != nil {
		logrus.Errorf("Error generating random bytes: %s", err)
		return nil, length, err
	}
	logrus.Debugf("Generated %d random bytes", length)

	return b, length, nil
}
