package randomnumbergenerator

import (
	"testing"

	"github.com/stretchr/testify/assert"
)

const seed = 12345

func TestNewRandomNumberGenerator(t *testing.T) {
	rng := NewRandomNumberGenerator(10, 20, seed, 0)

	assert.Equal(t, 10, rng.randomBytesMinimumLength)
	assert.Equal(t, 20, rng.randomBytesMaximumLength)
	assert.Equal(t, int64(seed), rng.seed)
}

func TestInitializeRandomGenerator(t *testing.T) {
	rng := NewRandomNumberGenerator(10, 20, seed, 0)

	err := rng.InitializeRandomGenerator()
	assert.NoError(t, err)
	assert.NotNil(t, rng.randomGenerator)
}

func TestGenerateRandomBytes(t *testing.T) {
	rng := NewRandomNumberGenerator(10, 20, seed, 0)
	err := rng.InitializeRandomGenerator()

	assert.NoError(t, err)

	randomBytes, length, err := rng.GenerateRandomBytes()
	assert.NoError(t, err)

	if len(randomBytes) < 10 || len(randomBytes) > 20 {
		t.Errorf("Expected randomBytes length to be between 10 and 20, got %d", len(randomBytes))
	}

	assert.Equal(t, length, len(randomBytes))
}

func TestSeedPersistenceGenerateRandomBytes(t *testing.T) {
	rng := NewRandomNumberGenerator(10, 10, seed, 0)
	err := rng.InitializeRandomGenerator()
	assert.NoError(t, err)

	randomBytes1, length, err := rng.GenerateRandomBytes()
	assert.NoError(t, err)
	assert.Equal(t, 10, len(randomBytes1))
	assert.Equal(t, length, len(randomBytes1))

	err = rng.InitializeRandomGenerator()
	assert.NoError(t, err)

	randomBytes2, length, err := rng.GenerateRandomBytes()
	assert.NoError(t, err)

	assert.Equal(t, 10, len(randomBytes2))
	assert.Equal(t, length, len(randomBytes2))

	// Compare the two random byte slices, they should be identical, as both times the rng is initialized with the same seed
	for i := 0; i < len(randomBytes1); i++ {
		assert.Equal(t, randomBytes1[i], randomBytes2[i])
	}
}

func TestGenerateRandomBytesInNeededSize(t *testing.T) {
	rng := NewRandomNumberGenerator(9000, 12000, seed, 0)
	err := rng.InitializeRandomGenerator()
	assert.NoError(t, err)

	for i := 0; i < 100; i++ {
		randomBytes, length, err := rng.GenerateRandomBytes()
		assert.NoError(t, err)

		assert.Equal(t, length, len(randomBytes))
	}
}
